
from django.db import models
import pandas as pd
from pandas import DataFrame as DF



class Country(models.Model):
    ''' A instance of this class represents a country in the world using the ISO 3166-1 standard '''
    alpha2 = models.CharField(max_length=2, null=True)
    alpha3 = models.CharField(max_length=3, unique=True)
    num = models.PositiveIntegerField(unique=True)
    name = models.CharField(max_length=250)

    @classmethod
    def create_objs(cls, update=False):
        url = 'https://www.iban.com/country-codes'
        df = pd.read_html(url)[0]
        df.columns = ['name', 'alpha2', 'alpha3', 'num']
        
        for tup in df.itertuples():
            alpha2 = None if pd.isna(tup.alpha2) else tup.alpha2
            goc_method = cls.objects.updated_or_create if update else cls.objects.get_or_create
            goc_method(
                alpha3 = tup.alpha3,
                defaults = dict(
                    alpha2=alpha2, num=tup.num, name=tup.name
                )
            )

    def __str__(self):
        return f'{self.alpha3}'


class Subdivision(models.Model):
    ''' Represents a country dub division using the ISO 3166-2 standard '''
    country = models.ForeignKey('Country', on_delete=models.PROTECT)
    iso_code = models.CharField(max_length=10, unique=True)
    name = models.CharField(max_length=250)
    category = models.CharField(max_length=50)


    @classmethod
    def create_objs_from_country(cls, country):
        ''' Method takes a coutnry object and creates all Subdivisions for that country '''
        url = f'https://en.wikipedia.org/wiki/ISO_3166-2:{country.alpha2}'
        dfs = pd.read_html(url)
        df = DF()
        for tdf in dfs:
            if 'Code' in tdf.columns and 'Subdivision category' in tdf.columns: 
                df = tdf
                break 
        if df.empty: return
        df.columns = ['code', 'name', 'category']
        for tup in df.itertuples():
            cls.objects.get_or_create(
                iso_code = tup.code,
                defaults = dict(
                    name = tup.name, category = tup.category, country=country
                )
            )

    def __str__(self):
        return f'{self.iso_code} | {self.name}'



from django.db import connection
import string
import random


class PasswordUtils:
    AMBIGUOUS_CHARS = ['0', 'O', 'l', 'I', 'o']

    def __init__(self, valid_symbols=None):
        if valid_symbols == None:
            self.valid_symbols = ['!','$','#','^']
        else: 
            self.valid_symbols = valid_symbols

    def check_password(
        self, password, min_length=12, must_have_symbol=True, must_have_caps=True, must_have_digit=True
    ):
        pset = set(password)
        assert len(password) >= min_length, f"Password must be at least {min_length} characters"
        assert len(pset.intersection(string.ascii_lowercase)) > 0, f"Password must contain at least 1 lowercase letter"
        if must_have_symbol:
            assert len(pset.intersection(self.valid_symbols)) > 0, f"Password must contain at least 1 special character {self.valid_symbols}"
        if must_have_caps:
            assert len(pset.intersection(string.ascii_uppercase)) > 0, f"Password must contain at least 1 uppercase letter"
        if must_have_digit:
            assert len(pset.intersection(string.digits)) > 0, f"Password must contain at least 1 digit {string.digits}"
        return True


    def generate_password(self, N=15, include_symbols=True, remove_ambiguous=True):
        all = string.ascii_letters + string.digits 
        if include_symbols:
            all += ''.join(self.valid_symbols) * 3
        if remove_ambiguous:
            for bad in self.AMBIGUOUS_CHARS:
                all = all.replace(bad,'')
        return ''.join(random.sample(all, N))




def create_view_from_qs(qs, view_name, sql_permissions=None):
    """Utility function to create a DB view from a django queryset.

    Args:
        qs (django.db.QuerySet): The queryset to be translated into a view
        view_name (str): View name to be created/updated
        sql_permissions (str, optional): SQL to be executed after the view is created, 
            intended for setting permissions but could be any raw SQL. 
            i.e. ALTER TABLE public.{view_name} OWNER TO postgres;
    """    
    qstr, params = qs.query.sql_with_params()
    drop_qstr = f''' DROP VIEW IF EXISTS "{view_name}" '''
    qstr = f''' CREATE OR REPLACE VIEW "{view_name}" AS {qstr} '''
    with connection.cursor() as cursor:
        cursor.execute(drop_qstr)
        cursor.execute(qstr, params)
    if sql_permissions:
        with connection.cursor() as cursor:
            cursor.execute(sql_permissions)





